#
# $Id: Tools.pm,v 1.5 2007/04/02 16:48:22 gomor Exp $
#
package Net::Frame::Tools;
use strict;
use warnings;

our $VERSION = '1.00';

1;

__END__

=head1 NAME

Net::Frame::Tools - useful network utilities created using Net::Frame

=head1 DESCRIPTION

This package installs various Perl programs. All these programs are written using Net::Frame::* modules, and are named nf-*.pl.

Each program does a specific task, using UNIX spirit.

=head1 AUTHOR

Patrice E<lt>GomoRE<gt> Auffret

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2006-2007, Patrice E<lt>GomoRE<gt> Auffret

You may distribute this module under the terms of the Artistic license.
See LICENSE.Artistic file in the source distribution archive.

=cut
